local MAX_LEVEL = 90

local tblOops = {}      -- Primary warnings: bad stats, missing auras
local tblOops2 = {}     -- Secondary warnings: missing gems, missing enchants
local tblOops3 = {}     -- Secondary warnings: missing gems, missing enchants
local tooltip = ''

local PLAIN_LETTER = 8383   -- For use in generating simple tooltips
local LibWeagleTooltip = LibStub("LibWeagleTooltip-2.1")

------------------------------------------------------------------
--      iter_cycle
do
    local function next (self)
        local ret = self.values[self.cur]
        self.cur = self.cur + 1
        if self.cur>#self.values then
            self.cur = 1
        end
        return ret
    end
   
    function iter_cycle (...)
        local obj = {
            values = {...},
            cur = 1,
            next = next,
        }
      
        return obj
    end   
end

------------------------------------------------------------------

local function VerifyItemStats (itemLink)
    -- Check against the bad stats table, return false for a bad item.
    local class = UnitClass('player')
    local spec = GetSpecialization('player')
    local tblStats = GetItemStats (itemLink)
    
    if tblWrongStats[class]==nil  or  spec==nil then 
        return true
    end
    
    if tblStats==nil then tblStats={} end -- Just to prevent a stupid login error, really
    for stat,statVal in pairs(tblStats) do
        for i,wrongStat in pairs(tblWrongStats[class][spec]) do
            if stat==wrongStat then
                return false
            end
        end
    end
    
    return true
end

------------------------------------------------------------------

local function VerifyItemType (itemLink)
    -- For high level toons, also check ilvl and quality. The ilvl threshold is pretty arbitrary.
    -- Return false for a bad item.
    if UnitLevel('player') == MAX_LEVEL then
        local _,_,qual, ilvl = GetItemInfo (itemLink)
        if qual==nil or ilvl==nil then      -- This seems to happen occasionally... must be a bug of some sort of while logging on...
            return true
        end
        if qual<=1 or ilvl<=300 then
            return false
        end
    end
    
    return true
end


------------------------------------------------------------------

local function CountSockets_Total (itemLink)
    if itemLink==nil then return 0 end
    local tblStats = GetItemStats (itemLink)
    local ret = 0

    if tblStats==nil then tblStats={} end -- Just to prevent a stupid login error, really
    for k,v in pairs(tblStats) do
        if k:match('SOCKET') then 
            ret = ret+v
        end
    end
    
    return ret
end

------------------------------------------------------------------

local function CountSockets_Gemmed (itemLink)
    if itemLink==nil then return 0 end
    local itemId, enchantId, gem1, gem2, gem3, gem4 = itemLink:match("item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+)")
    
    local ret = 0
    for i,v in ipairs({gem1,gem2,gem3,gem4}) do
        if v~='0' then
            ret = ret+1
        end
    end
    
    return ret
end

------------------------------------------------------------------

local function StringHasBadStat (str)
    if str==nil then 
        return false
    end
    
    local class = UnitClass('player')
    local spec = GetSpecialization('player')
    for i,stat in pairs(tblWrongStats[class][spec]) do
        if str:match (tblStatNames[stat]) then
            return true
        end
    end
    
    return false
end

------------------------------------------------------------------

local function VerifySocketAndEnchantStats (itemLink)
    -- Generate a tooltip of a plain letter (statless, cached item) with the same enchant and gems as this item
    local itemId, enchantId, gem1, gem2, gem3, gem4 = itemLink:match("item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+)")
    local tooltip = ("item:%i:%i:%i:%i:%i:%i"):format(PLAIN_LETTER, enchantId, gem1, gem2, gem3, gem4)

    -- Get the enchant from line 2 of the tooltip, and up to 4 gems from the next 4 lines
    local tmp = {}  -- This table will contain the enchant text, followed by up to 4 gem texts
    for i=2,6 do
        tmp[i-1] =  LibWeagleTooltip:GetTooltipLine(tooltip, i)
    end
    
    for k,v in pairs(tmp) do
        if StringHasBadStat(v) then
            return false
        end
    end

    return true
end

------------------------------------------------------------------

local function VerifyItemSockets (itemLink)
    local _,_,_, ilvl,_,_,_,_,equipSlot = GetItemInfo(itemLink)
    
    if ilvl==nil or ilvl < g_SanityCheck_config['GemMinLvl'] then
        return true
    else
        return (CountSockets_Total(itemLink) <= CountSockets_Gemmed(itemLink))
    end
end

------------------------------------------------------------------

local function VerifyBeltBuckle ()
    -- Only if the belt is gemmed to begin with
    local itemLink = GetInventoryItemLink ('player', INVSLOT_WAIST)
    if not itemLink then return true end
    
    local _,_,_, ilvl = GetItemInfo(itemLink)
    
    if ilvl==nil or ilvl < g_SanityCheck_config['EnchMinLvl'] then
        return true
    end
    
    return (CountSockets_Total(itemLink)+1 == CountSockets_Gemmed(itemLink))
end

------------------------------------------------------------------

local function VerifyItemEnchant (itemLink)
    local _,_,_, ilvl= GetItemInfo(itemLink)
    
    if ilvl==nil or ilvl < g_SanityCheck_config['EnchMinLvl'] then
        return true
    else
        local itemId, enchantId = itemLink:match("item:(%d+):(%d+)")
        return enchantId~='0'
    end
end

------------------------------------------------------------------

local function VerifyRoleAndSpec ()
    local class = UnitClass('player')
    local spec = GetSpecialization('player')
    local role = UnitGroupRolesAssigned('player')
    
    if tblRoles[class]==nil or spec==nil or UnitGroupRolesAssigned('player')=='NONE' then return true,'' end
    
    if tblRoles[class][spec] == role then
        return true, ''
    else
        local _, spec = GetSpecializationInfo(GetSpecialization())
        return false, 'Are you sure you want to '..tblVerbs[role]..' in '..spec..' spec?'
    end
end

------------------------------------------------------------------

function CheckInventory ()
    if tblWrongStats[UnitClass('player')]==nil then return end
            
    for slot,slotName in pairs(tblSlots) do
        local link = GetInventoryItemLink ('player', slot)
        if link~=nil then
            if not VerifyItemStats(link) or not VerifyItemType(link) then
                table.insert (tblOops, 'Bad item: '..slotName)
            end
            if not VerifyItemSockets (link) then
                table.insert (tblOops2, 'Missing gems: '..slotName)
            end
            if not VerifySocketAndEnchantStats (link) then
                table.insert (tblOops2, 'Bad gems/ enchant: '..slotName)
            end
            
            if tblSlots_Enchantable[slot] and (not VerifyItemEnchant(link)) then
                table.insert (tblOops2, 'Missing enchant: '..slotName)
            end
        else
            -- Mention if the item is missing - unless it's an offhand, and there's a 2H equipped
            -- Low level toons can get away with empty slots
            if (slot==INVSLOT_OFFHAND and Has2HanderEquipped()) or (UnitLevel('player')<60) then
                -- All is well
            else
                table.insert (tblOops, 'Missing item: '..slotName)
            end
        end
    end
    
    if not VerifyBeltBuckle() then
        table.insert (tblOops2, 'Hey, how about a belt buckle and an extra gem?')
    end
end

------------------------------------------------------------------

function CheckAuras ()
    local class = UnitClass('player')
    local spec = GetSpecialization('player')
    
    -- Personal auras and stuff
    if tblVerifyAuras[class]==nil or spec==nil then return end
    
    local funcs
    local tmp = tblVerifyAuras[class][spec]
    if type(tmp)=='function' then
        funcs = {tmp}
    else
        funcs = tmp
    end
    
    for i,func in ipairs(funcs) do
        ret, msg = func()
        if not ret then
            table.insert (tblOops, msg)
        end
    end
    
    -- Raid buffs
    if tblVerifyRaidBuffs[class]==nil then return end
    
    ret, msg = tblVerifyRaidBuffs[class]()
    if not ret then
        table.insert (tblOops3, msg)
    end
    
    -- Role
    ret, msg = VerifyRoleAndSpec()
    if not ret then
        table.insert (tblOops, msg)
    end    
end

------------------------------------------------------------------

local function CreateLightFrame (name, parent)
    local f = CreateFrame ("Frame", name, parent)
    f:SetBackdrop ({
        bgFile = "Interface\\Addons\\SanityCheck\\red.tga",  
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
        edgeSize = 8,
    })

    f.lbl = f:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    f.lbl:SetPoint ("CENTER", f, "CENTER")
    f.lbl:SetSize (100,30)
    f.lbl:SetTextColor (1.0,1.0,1.0)
    f.lbl:SetJustifyH ("CENTER")
    
    f:SetScript("OnEnter", function(self, motion, ...)
        GameTooltip:SetOwner (self, "ANCHOR_TOP")
        self:GenerateTooltip ()
        GameTooltip:Show ()
    end)
    f:SetScript("OnLeave", function(self, motion, ...)
        GameTooltip:Hide ()
    end)

    return f
end

------------------------------------------------------------------


local function BuildGui ()
    -- Primary warning light
    do
        local f = CreateLightFrame ('frmSanityCheck', UIParent)
        f:SetPoint ("CENTER", UIParent, "CENTER", 0, 100)
        f:SetSize (110,40)
        
        f.elapsed = 0
        f.itBlinker = iter_cycle ({1.0, 0.0, 0.0, 0.9}, {0.3, 0.0, 0.0, 0.65} )
        f.blinkInterval = 0.55
        f.blink = true
        function f:Light ()
            --self:SetBackdropColor (1.0, 1.0, 1.0, 0.75)
            self.lbl:SetText ("OMG N00b!")
            if self.isOn==false then
                self.blink = true
            end
            self.isOn = true
        end
        function f:Dark ()
            self:SetBackdropColor (0.0, 0.0, 0.0, 0.5)
            self.lbl:SetText ("You're awesome.")
            self.isOn = false
        end

        function f:GenerateTooltip ()
            if self.isOn then
                GameTooltip:AddDoubleLine ('SanityCheck', 'Right-click to stop blinking', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
            
                if #tblOops>0 then
                    for k,v in pairs(tblOops) do
                        GameTooltip:AddLine (v, 1.0, 0.5, 0.5)
                    end
                end
            else
                GameTooltip:AddLine ('SanityCheck', 1.0, 1.0, 1.0)
                GameTooltip:AddLine ('Your gear and buffs appear to be fine!', 1.0, 0.5, 0.5)
            end
        end

        f:SetMovable (true)
        f:RegisterForDrag ("LeftButton")
        f:SetUserPlaced (true)
        f:SetClampedToScreen (true)
        f:EnableMouse (true)
        f:SetScript("OnMouseDown", function(self, button)
            if button=='LeftButton' then
                self:StartMoving ()
                GameTooltip:Hide()
            elseif button=='RightButton' then
                self.blink = false
            end
        end)
        f:SetScript("OnMouseUp", function(self, button)
            self:StopMovingOrSizing ()
        end)

        f:Dark()
    end

    -- Secondary warning light
    do
        local f = CreateLightFrame ('frmSanityCheck_Secondary', frmSanityCheck)
        f:SetPoint ("LEFT", frmSanityCheck, "RIGHT")
        f:SetSize (20,40)
        
        function f:Light ()
            self:SetBackdropColor (0.9, 0.65, 0.0, 0.85)
            self.lbl:SetText ('!')
            self.isOn = true
        end
        function f:Dark ()
            self:SetBackdropColor (0.0, 0.0, 0.0, 0.5)
            self.lbl:SetText ('')
            self.isOn = false
        end
        f:Dark()
        
        function f:GenerateTooltip ()
            if self.isOn then
                GameTooltip:AddDoubleLine ('SanityCheck', 'Secondary Warnings', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
            
                for k,v in pairs(tblOops2) do
                    GameTooltip:AddLine (v, 1.0, 0.5, 0.5)
                end
            else
                GameTooltip:AddDoubleLine ('SanityCheck', 'Secondary Warnings', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
                GameTooltip:AddLine ('You seem to have all your gems and enchants!', 1.0, 0.5, 0.5)
            end
        end
       
        frmSanityCheck.frmSec = f
    end

    -- Tertiary warning light
    do
        local f = CreateLightFrame ('frmSanityCheck_Tertiary', frmSanityCheck)
        f:SetPoint ("BOTTOMRIGHT", frmSanityCheck, "BOTTOMLEFT")
        f:SetSize (20,20)
        
        function f:Light ()
            self:SetBackdropColor (0.6, 0.75, 1.0, 0.85)
            self.isOn = true
        end
        function f:Dark ()
            self:SetBackdropColor (0.0, 0.0, 0.0, 0.5)
            self.isOn = false
        end
        f:Dark()
        
        function f:GenerateTooltip ()
            if self.isOn then
                GameTooltip:AddDoubleLine ('SanityCheck', 'Even More Warnings', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
            
                for k,v in pairs(tblOops3) do
                    GameTooltip:AddLine (v, 1.0, 0.5, 0.5)
                end
            else
                GameTooltip:AddDoubleLine ('SanityCheck', 'Even More Warnings', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
                GameTooltip:AddLine ('Nothing to see here, move along now!', 1.0, 0.5, 0.5)
            end
        end
       
        frmSanityCheck.frmTer = f
    end

    --  Configure button
    do
        local f = CreateFrame ("Frame", nil, frmSanityCheck)
        f:SetPoint ("TOPRIGHT", frmSanityCheck, "TOPLEFT")
        f:SetSize (20,20)
        f:SetBackdrop ({
            edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
            edgeSize = 10,
        })
        
        local btn = CreateFrame ("Button", nil, f, "UIPanelButtonTemplate")
        btn:SetAllPoints (f)
        --btn:SetText ("@")
        btn:SetFrameStrata("LOW")
        --btn:SetNormalTexture ("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up")
        --btn:SetPushedTexture ("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down")
        --btn:SetHighlightTexture ("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")
        
        btn:SetScript("OnEnter", function(self, motion, ...)
            GameTooltip:SetOwner (self, "ANCHOR_TOP")
            GameTooltip:AddLine ('Configure', 1.0, 1.0, 1.0)
            GameTooltip:Show ()
        end)
        btn:SetScript("OnLeave", function(self, motion, ...)
            GameTooltip:Hide ()
        end)
        btn:SetScript("OnClick", function (self,...)
            frmSanityCheckConfig:Show()
        end)

        frmSanityCheck.btnConfig = btn
    end
end

------------------------------------------------------------------
--  Config frame

local function MakeConfigFrame ()
    local function MakeOptionLabel (frm, txt)
        local lbl = frm:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
        lbl:SetTextColor (1,1,1)
        lbl:SetText (txt)
        lbl:SetJustifyH ("RIGHT")
        
        return lbl
    end
    
    local function MakeOptionTextbox (frm)
        local txt = CreateFrame ("EditBox", nil, frm)
        txt:SetBackdrop ({
            bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
            edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
            tile = true,
            tileSize = 32,
            edgeSize = 8,
        })
        txt:SetTextColor (1,1,1)
        txt:SetFont ('Fonts\\ARIALN.TTF', 14, "")
        txt:SetAutoFocus (false)
        txt:SetTextInsets (4, 4, 0, 0)
        txt:SetScript ("OnEscapePressed", txt.ClearFocus)
        txt:SetNumeric (true)
        
        return txt
    end
    
    -- Default values
    if g_SanityCheck_config==nil then
        g_SanityCheck_config = {
            EnchMinLvl = 483,
            GemMinLvl = 483,
        }
    end
    
    -- Make frame
    local W, H = 440, 140
    local f = CreateFrame ("Frame", 'frmSanityCheckConfig', frmSanityCheck)
    f:SetPoint ("CENTER", UIParent, "CENTER")
    f:SetSize (W,H)
    f:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
        edgeSize = 16,
    })
    
    -- Title
    f.title = f:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    f.title:SetPoint ("TOPLEFT", f, "TOPLEFT", 0, -4)
    f.title:SetPoint ("TOPRIGHT", f, "TOPRIGHT", 0, -4)
    f.title:SetHeight(20)
    f.title:SetText ('SanityCheck Options')
    
    -- X button
    f.btnX = CreateFrame ("Button", nil, f, "UIPanelCloseButton")
    f.btnX:SetPoint("TOPRIGHT", f, "TOPRIGHT")
    f.btnX:SetSize(24,24)

    -- Options 1: ilvl that requires an enchant    
    f.lblEnchMinLvl = MakeOptionLabel(f, "Don't require enchants on items lower than ilvl:")
    f.lblEnchMinLvl:SetPoint ("TOPRIGHT", f, "TOPRIGHT", -104, -25)
    f.lblEnchMinLvl:SetSize (W-54, 34)
    
    f.txtEnchMinLvl = MakeOptionTextbox (f)
    f.txtEnchMinLvl:SetPoint ("TOPRIGHT", f, "TOPRIGHT", -38, -31)
    f.txtEnchMinLvl:SetSize (46, 22)
    f.txtEnchMinLvl:SetNumber (g_SanityCheck_config['EnchMinLvl'])
    
    -- Options 1: ilvl that requires gems    
    f.lblGemMinLvl = MakeOptionLabel(f, "Don't require gems in items lower than ilvl:")
    f.lblGemMinLvl:SetPoint ("TOPRIGHT", f, "TOPRIGHT", -104, -55)
    f.lblGemMinLvl:SetSize (W-54, 34)
    
    f.txtGemMinLvl = MakeOptionTextbox (f)
    f.txtGemMinLvl:SetPoint ("TOPRIGHT", f, "TOPRIGHT", -38, -61)
    f.txtGemMinLvl:SetSize (46, 22)
    f.txtGemMinLvl:SetNumber (g_SanityCheck_config['GemMinLvl'])
    
    
    -- OK/ Cancel buttons    
    f.btnCancel = CreateFrame ("Button", nil, f, "UIPanelButtonTemplate")
    f.btnCancel:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -14, 10)
    f.btnCancel:SetSize(80,24)
    f.btnCancel:SetText ("Cancel")
    f.btnCancel:SetScript ("OnClick", function (self,...) frmSanityCheckConfig:Hide() end)
    
    f.btnOk = CreateFrame ("Button", nil, f, "UIPanelButtonTemplate")
    f.btnOk:SetPoint("RIGHT", f.btnCancel, "LEFT", -7, 0)
    f.btnOk:SetSize(80,24)
    f.btnOk:SetText ("OK")
    f.btnOk:SetScript ("OnClick", function (self,...)
        g_SanityCheck_config['EnchMinLvl'] = frmSanityCheckConfig.txtEnchMinLvl:GetNumber()
        g_SanityCheck_config['GemMinLvl'] = frmSanityCheckConfig.txtGemMinLvl:GetNumber()
        frmSanityCheckConfig:Hide()
    end)
    
    f:Hide()
end

------------------------------------------------------------------
--  Events and such
local function RegisterEvents ()
    local f = frmSanityCheck
    
    f:RegisterEvent ("UNIT_INVENTORY_CHANGED")
    f:RegisterEvent ("PLAYER_SPECIALIZATION_CHANGED")
    f:RegisterEvent ("UNIT_AURA")
    f:RegisterEvent ("UNIT_PET")
    f:RegisterEvent ("PLAYER_ENTERING_WORLD")
    f:RegisterEvent ("GROUP_ROSTER_UPDATE")
    f:SetScript ("OnEvent", function  (self, event, GUID, ...)
        if (GUID == 'player'  or  event=='PLAYER_ENTERING_WORLD'  or  event=='GROUP_ROSTER_UPDATE')  and  (not UnitAffectingCombat('player')) then
            tblOops = {}
            tblOops2 = {}
            tblOops3 = {}
            
            CheckInventory ()
            CheckAuras ()
            
            if #tblOops > 0 then
                frmSanityCheck:Light ()
            else
                frmSanityCheck:Dark ()
            end

            if #tblOops2 > 0 then
                frmSanityCheck.frmSec:Light ()
            else
                frmSanityCheck.frmSec:Dark ()
            end

            if #tblOops3 > 0 then
                frmSanityCheck.frmTer:Light ()
            else
                frmSanityCheck.frmTer:Dark ()
            end
        end
    end)
    f:SetScript ("OnUpdate", function (self, elapsed, ...)
        if self.isOn==false then return end
        
        self.elapsed = self.elapsed + elapsed
        if self.elapsed > self.blinkInterval then
            self.elapsed = 0
            if self.blink then
                self:SetBackdropColor (unpack(self.itBlinker:next()))
            else
                self:SetBackdropColor (unpack(self.itBlinker.values[1]))
            end
        end
    end)
end


------------------------------------------------------------------
------------------------------------------------------------------

BuildGui ()
local starter = CreateFrame ("Frame")
starter:RegisterEvent ("PLAYER_ENTERING_WORLD")
starter:SetScript ("OnEvent", function  (self, e, ...)
    MakeConfigFrame ()
    RegisterEvents ()
    frmSanityCheck:GetScript ('OnEvent') (frmSanityCheck, 'PLAYER_ENTERING_WORLD')
end)